import { Body, Controller, Get, Put, Request, UseGuards } from '@nestjs/common';
import { AuthGuard } from '@nestjs/passport';
import { ApiBearerAuth, ApiBody, ApiOperation, ApiTags } from '@nestjs/swagger';
import { Serealize } from '../interceptors/serialize.interceptor';
import { InstitucionUsuarioService } from './institucion-usuario.service';
import { ValidarUsuarioService } from '../validar-usuario/validar-usuario.service';
import { Operador } from '../operador/entity/operador.entity';
import { Usuario } from '../usuario/entity/usuario.entity';
import { UpdateInstitucionUsuarioInputDto } from './dto/input/update.dto';
import { MessageOutputDto } from '../dto/output/message.dto';
import { InstitucionUsuarioMinOutputDto } from './dto/output/institucion-usuario-min.dto';

@Controller('institucion-usuario')
@ApiTags('institucion-usuario')
export class InstitucionUsuarioController {
  constructor(
    private institucionUsuarioService: InstitucionUsuarioService,
    private validarUsuarioService: ValidarUsuarioService,
  ) {}

  @Serealize(InstitucionUsuarioMinOutputDto)
  @Get()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description:
      'Todas las instituciones activas a las que pertenece un usuario.',
  })
  @ApiBearerAuth('jwt')
  get(@Request() req) {
    const usuario: Usuario = req.user.usuario;

    this.validarUsuarioService.validarUsuario(usuario);
    return this.institucionUsuarioService.findFullInfoAllByIdUsuario(usuario);
  }

  @Serealize(MessageOutputDto)
  @Put('desactivar-cuentas')
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Desactivar todas las cuentas de la institución de un admin.',
  })
  @ApiBearerAuth('jwt')
  desactivarCuentas(@Request() req) {
    const admin: Operador = req.user.operador;

    this.validarUsuarioService.validarAdmin(admin);
    return this.institucionUsuarioService.desactivarCuentas(admin);
  }

  @Serealize(MessageOutputDto)
  @Put()
  @UseGuards(AuthGuard('jwt'))
  @ApiOperation({
    description: 'Actualizar la información de una institución usuario.',
  })
  @ApiBearerAuth('jwt')
  @ApiBody({
    description: 'Variables que necesita el endpoint.',
    examples: {
      ejemplo: { value: { activo: true, id_institucion_usuario: 1 } },
    },
  })
  update(@Request() req, @Body() body: UpdateInstitucionUsuarioInputDto) {
    const admin: Operador = req.user.operador;

    this.validarUsuarioService.validarAdminOperador(admin);
    return this.institucionUsuarioService.update(body);
  }
}
